#ifndef QUEUE_TS_H_
#define QUEUE_TS_H_

#include <pthread.h>
#include <stdlib.h>
#include <string.h>

#include "Queue_TSData.h"

// these two values are used
// for the timeout argument to queueDequeue

// do not block, return NULL immediately if the queue is empty
#define QUEUE_NOWAIT 0 
// wait forever if the queue is empty
#define QUEUE_BLOCK -1

/*
 * Allocate a queue struct and initialize its data memebers
 */
struct Queue* queueInit();

/*
 * Add the data to the queue
 * This method should dynamically allocate memory to store the data in
 * the queue.  The data passed in by the user should NOT be directly
 * stored in the queue.
 * 
 * queue is the Queue were this data will be added
 * data is a pointer to the data to be added
 * len is the length of the data to be added
 * 
 * return 1 on success, 0 on error
 */
int queueEnqueue(struct Queue *psQueue, void *pData, int len);

/*
 * Remove data from the queue
 * This method should return a pointer to the data element at the head of the
 * queue.  It is the USER'S repsonibility to deallocate this memory.
 * This function should deallocate the QueueItem.
 * 
 * queue is the Queue 
 * len is the length of the data that is returned
 * timeout denotes how long to wait if the queue is empty. This is given in
 * milliseconds.
 * 
 * return a pointer to the data on success, NULL on error
 */
void* queueDequeue(struct Queue *psQueue, int *pLen, int timeout);

/*
 * Deallocate the queue and all the data it holds.
 * 
 * queue is the Queue
 */
void queueDestroy(struct Queue *psQueue);

/*
 * Return the data item at the head of the queue but DO NOT remove it
 * from the queue.  This is similar to top() when using a stack.
 * 
 * queue is the Queue
 * len is the length of the data returned
 * 
 * return a pointer to the data on success, NULL if the queue is empty
 */
 void* queueHead(struct Queue *psQueue, int *pLen);
 
/*
 * Return 1 if the queue contains data, 0 if the queue is empty.
 * 
 * queue is the Queue
 */
int queueProbe(struct Queue *psQueue);

#endif /*QUEUE_TS_H_*/
